@@ -61,12 +61,14 @@ materialization_endpoints_v3 = {
     "join_query": mat_v3_api + "/datastack/{datastack_name}/version/{version}/query",
     "table_count": mat_v2_api
     + "/datastack/{datastack_name}/version/{version}/table/{table_name}/count",
-    "versions": mat_v2_api + "/datastack/{datastack_name}/versions",
-    "version_metadata": mat_v2_api + "/datastack/{datastack_name}/version/{version}",
+    "versions": mat_v3_api + "/datastack/{datastack_name}/versions",
+    "version_metadata": mat_v3_api + "/datastack/{datastack_name}/version/{version}",
     "tables": mat_v2_api + "/datastack/{datastack_name}/version/{version}/tables",
-    "metadata": mat_v2_api
+    "metadata": mat_v3_api
     + "/datastack/{datastack_name}/version/{version}/table/{table_name}/metadata",
-    "versions_metadata": mat_v2_api + "/datastack/{datastack_name}/metadata",
+    "all_tables_metadata": mat_v3_api
+    + "/datastack/{datastack_name}/version/{version}/tables/metadata",
+    "versions_metadata": mat_v3_api + "/datastack/{datastack_name}/metadata",
     "ingest_annotation_table": mat_v2_api
     + "/materialize/run/ingest_annotations/datastack/{datastack_name}/{table_name}",
     "segmentation_metadata": mat_v3_api
@@ -74,6 +76,17 @@ materialization_endpoints_v3 = {
     "live_live_query": mat_v3_api + "/datastack/{datastack_name}/query",
     "lookup_supervoxel_ids": mat_v2_api
     + "/materialize/run/lookup_svid/datastack/{datastack_name}/{table_name}",
+    "get_views": mat_v3_api + "/datastack/{datastack_name}/version/{version}/views",
+    "get_view_metadata": mat_v3_api
+    + "/datastack/{datastack_name}/version/{version}/views/{view_name}/metadata",
+    "view_query": mat_v3_api
+    + "/datastack/{datastack_name}/version/{version}/views/{view_name}/query",
+    "view_schema": mat_v3_api
+    + "/datastack/{datastack_name}/version/{version}/views/{view_name}/schema",
+    "view_schemas": mat_v3_api
+    + "/datastack/{datastack_name}/version/{version}/views/schemas",
+    "unique_string_values": mat_v3_api
+    + "/datastack/{datastack_name}/table/{table_name}/unique_string_values",
 }
 
 materialization_api_versions = {
@@ -115,6 +128,7 @@ infoservice_api_versions = {1: infoservice_endpoints_v1, 2: infoservice_endpoint
 pcg_common = "{cg_server_address}/segmentation"
 chunkedgraph_endpoints_common = {
     "get_api_versions": pcg_common + "/api/versions",
+    "get_version": pcg_common + "/api/version",
     "info": pcg_common + "/table/{table_id}/info",
 }
 
@@ -187,6 +201,8 @@ schema_v2 = "{emas_server_address}/schema/api/v2"
 schema_endpoints_v2 = {
     "schema": schema_v2 + "/type",
     "schema_definition": schema_v2 + "/type/{schema_type}",
+    "schema_definition_multi": schema_v2 + "/types",
+    "schema_definition_all": schema_v2 + "/types_all",
 }
 
 schema_api_versions = {1: schema_endpoints_v1, 2: schema_endpoints_v2}
@@ -203,6 +219,11 @@ jsonservice_endpoints_v1 = {
     "upload_state_w_id": json_v1 + "/post/{state_id}",
     "get_state": json_v1 + "/{state_id}",
     "get_state_raw": json_v1 + "/raw/{state_id}",
+    "get_properties": json_v1 + "/property/{state_id}/info",
+    "upload_properties": json_v1 + "/property/post",
+    "get_properties_raw": json_v1 + "/property/raw/{state_id}",
+    "upload_properties_w_id": json_v1 + "/property/post/{state_id}",
+    "get_version": json_v1 + "/version",
 }
 
 json_legacy = "{json_server_address}/nglstate"
@@ -249,6 +270,17 @@ l2cache_v1 = "{l2cache_server_address}/l2cache/api/v1"
 l2cache_endpoints_v1 = {
     "l2cache_data": l2cache_v1 + "/table/{table_id}/attributes",
     "l2cache_meta": l2cache_v1 + "/attribute_metadata",
+    "l2cache_table_mapping": l2cache_v1 + "/table_mapping",
 }
 
 l2cache_api_versions = {1: l2cache_endpoints_v1}
+
+# -------------------------------
+# ------ Neuroglancer endpoints
+# -------------------------------
+
+fallback_ngl_endpoint = "https://neuroglancer.neuvue.io/"
+ngl_endpoints_common = {
+    "get_info": "{ngl_url}/version.json",
+    "fallback_ngl_url": fallback_ngl_endpoint,
+}
