@@ -12,6 +12,16 @@ def format_precomputed_neuroglancer(objurl):
     return objurl_out
 
 
+def format_neuroglancer(objurl):
+    qry = urlparse(objurl)
+    if qry.scheme == "graphene" or "https":
+        return format_graphene(objurl)
+    elif qry.scheme == "precomputed":
+        return format_precomputed_neuroglancer(objurl)
+    else:
+        return format_raw(objurl)
+
+
 def format_precomputed_https(objurl):
     qry = urlparse(objurl)
     if qry.scheme == "gs":
@@ -33,6 +43,16 @@ def format_graphene(objurl):
         objurl_out = None
     return objurl_out
 
+
+def format_verbose_graphene(objurl):
+    qry = urlparse(objurl)
+    if qry.scheme == "http" or qry.scheme == "https":
+        objurl_out = f"graphene://middleauth+{objurl}"
+    elif qry.scheme == "graphene":
+        objurl_out = f"graphene://middleauth+{qry.netloc}{qry.path}"
+    return objurl_out
+
+
 def format_cloudvolume(objurl):
     qry = urlparse(objurl)
     if qry.scheme == "graphene":
@@ -47,19 +67,22 @@ def format_raw(objurl):
     return objurl
 
 
-# No reformatting
-output_map_raw = {}
+def format_cave_explorer(objurl):
+    qry = urlparse(objurl)
+    if qry.scheme == "graphene" or qry.scheme == "https":
+        return format_verbose_graphene(objurl)
+    elif qry.scheme == "precomputed":
+        return format_precomputed_neuroglancer(objurl)
+    else:
+        return None
 
-# Use precomputed://gs:// links for neuroglancer, but use precomputed://https://storage.googleapis.com links in cloudvolume
-output_map_precomputed = {
-    "raw": format_raw,
-    "cloudvolume": format_precomputed_https,
-    "neuroglancer": format_precomputed_neuroglancer,
-}
 
 # Use graphene://https:// links for both neuroglancer and cloudvolume
-output_map_graphene = {
+
+output_map = {
     "raw": format_raw,
-    "cloudvolume": format_graphene,
-    "neuroglancer": format_graphene,
+    "cloudvolume": format_cloudvolume,
+    "neuroglancer": format_neuroglancer,
+    "cave_explorer": format_cave_explorer,
+    "cave-explorer": format_cave_explorer,
 }
