@@ -1,18 +1,21 @@
-from re import A, match
-from .conftest import test_info, TEST_LOCAL_SERVER, TEST_DATASTACK
-import pytest
-import responses
-import pytz
-import numpy as np
-from caveclient.endpoints import (
-    chunkedgraph_endpoints_v1,
-    chunkedgraph_endpoints_common,
-)
 import datetime
-import time
 import json
 from urllib.parse import urlencode
 
+import numpy as np
+import pytest
+import pytz
+import responses
+from responses.matchers import json_params_matcher
+
+from caveclient.base import ServerIncompatibilityError
+from caveclient.endpoints import (
+    chunkedgraph_endpoints_common,
+    chunkedgraph_endpoints_v1,
+)
+
+from .conftest import TEST_LOCAL_SERVER, test_info
+
 
 def binary_body_match(body):
     def match(request_body):
@@ -35,7 +38,6 @@ def package_timestamp(timestamp, name="timestamp"):
 
 
 class TestChunkedgraph:
-
     _default_endpoint_map = {
         "cg_server_address": TEST_LOCAL_SERVER,
         "table_id": test_info["segmentation_source"].split("/")[-1],
@@ -47,7 +49,7 @@ class TestChunkedgraph:
         url = chunkedgraph_endpoints_v1["get_roots"].format_map(endpoint_mapping)
         svids = np.array([97557743795364048, 75089979126506763], dtype=np.uint64)
         root_ids = np.array([864691135217871271, 864691135566275148], dtype=np.uint64)
-        now = datetime.datetime.utcnow()
+        now = datetime.datetime.now(datetime.timezone.utc)
         query_d = package_timestamp(now)
         qurl = url + "?" + urlencode(query_d)
         responses.add(
@@ -231,7 +233,7 @@ class TestChunkedgraph:
         endpoint_mapping = self._default_endpoint_map
         url = chunkedgraph_endpoints_v1["delta_roots"].format_map(endpoint_mapping)
 
-        now = datetime.datetime.utcnow()
+        now = datetime.datetime.now(datetime.timezone.utc)
         timestamp_past = now - datetime.timedelta(days=1)
         query_d = package_timestamp(timestamp_past, name="timestamp_past")
         query_d.update(package_timestamp(now, name="timestamp_future"))
@@ -365,6 +367,84 @@ class TestChunkedgraph:
         qlvl2_graph = myclient.chunkedgraph.level2_chunk_graph(root_id)
         assert np.all(qlvl2_graph == lvl2_graph)
 
+    @responses.activate
+    def test_get_lvl2subgraph_bounds(self, myclient):
+        endpoint_mapping = self._default_endpoint_map
+        root_id = 864691136812623475
+        endpoint_mapping["root_id"] = root_id
+        url = chunkedgraph_endpoints_v1["lvl2_graph"].format_map(endpoint_mapping)
+
+        lvl2_graph_list = [
+            [160032475051983415, 160032543771460210],
+            [160032475051983415, 160102843796161019],
+            [160032543771460210, 160032612490936816],
+            [160032543771460210, 160102912515637813],
+            [160032612490936816, 160032681210413593],
+            [160032612490936816, 160102981235115106],
+            [160032681210413593, 160032749929890185],
+            [160032681210413593, 160032749929890340],
+            [160032681210413593, 160103049954591386],
+            [160032749929890185, 160103118674068005],
+            [160032818649367090, 160103187393544707],
+            [160102843796161019, 160102912515637813],
+            [160102912515637813, 160102981235115106],
+            [160102981235115106, 160103049954591364],
+            [160102981235115106, 160103049954591386],
+            [160103049954591386, 160103118674068005],
+            [160103118674068005, 160103187393544707],
+            [160103187393544707, 160173556137722487],
+        ]
+
+        lvl2_graph = np.array(lvl2_graph_list, dtype=np.int64)
+
+        responses.add(
+            responses.GET,
+            json={"edge_graph": lvl2_graph_list},
+            url=url,
+            headers={"Used-Bounds": "True"},
+        )
+
+        bounds = np.array([[83875, 85125], [82429, 83679], [20634, 20884]])
+        qlvl2_graph = myclient.chunkedgraph.level2_chunk_graph(root_id, bounds=bounds)
+        assert np.all(qlvl2_graph == lvl2_graph)
+
+        # should fail when bounds are not used, but bounds were passed in
+        responses.add(
+            responses.GET,
+            json={"edge_graph": lvl2_graph_list},
+            url=url,
+            headers={"Used-Bounds": "False"},
+        )
+
+        with pytest.raises(ValueError):
+            qlvl2_graph = myclient.chunkedgraph.level2_chunk_graph(
+                root_id, bounds=bounds
+            )
+
+    @responses.activate
+    def test_lvl2subgraph_old_server_fails_bounds(self, old_chunkedgraph_client):
+        bounds = np.array([[83875, 85125], [82429, 83679], [20634, 20884]])
+        root_id = 864691136812623475
+        with pytest.raises(ServerIncompatibilityError):
+            old_chunkedgraph_client.chunkedgraph.level2_chunk_graph(
+                root_id, bounds=bounds
+            )
+        with pytest.raises(ServerIncompatibilityError):
+            old_chunkedgraph_client.chunkedgraph.level2_chunk_graph(root_id, bounds)
+
+        # should not fail even on old server when bounds are default value
+        # mock a response
+        endpoint_mapping = self._default_endpoint_map
+        endpoint_mapping["root_id"] = root_id
+        url = chunkedgraph_endpoints_v1["lvl2_graph"].format_map(endpoint_mapping)
+        responses.add(
+            responses.GET,
+            json={"edge_graph": []},
+            url=url,
+            headers={"Used-Bounds": "True"},
+        )
+        old_chunkedgraph_client.chunkedgraph.level2_chunk_graph(root_id, bounds=None)
+
     @responses.activate
     def test_get_remeshing(self, myclient):
         endpoint_mapping = self._default_endpoint_map
@@ -382,7 +462,7 @@ class TestChunkedgraph:
             responses.POST,
             status=200,
             url=url,
-            match=[responses.json_params_matcher({"new_lvl2_ids": chunkid_list})],
+            match=[json_params_matcher({"new_lvl2_ids": chunkid_list})],
         )
 
         myclient.chunkedgraph.remesh_level2_chunks(chunk_ids)
@@ -404,7 +484,7 @@ class TestChunkedgraph:
             status=200,
             url=url,
             json={"is_latest": is_latest_list},
-            match=[responses.json_params_matcher({"node_ids": root_id_list})],
+            match=[json_params_matcher({"node_ids": root_id_list})],
         )
 
         qis_latest = myclient.chunkedgraph.is_latest_roots(root_ids)
@@ -434,7 +514,7 @@ class TestChunkedgraph:
                 "864691136577570580": [864691136721486702, 864691133958789149],
             },
         }
-        now = datetime.datetime.utcnow()
+        now = datetime.datetime.now(datetime.timezone.utc)
         timestamp_past = now - datetime.timedelta(days=7)
 
         query_d = package_timestamp(timestamp_past, name="timestamp_past")
@@ -446,7 +526,7 @@ class TestChunkedgraph:
             status=200,
             url=qurl,
             json=id_map_str,
-            match=[responses.json_params_matcher({"root_ids": root_id_list})],
+            match=[json_params_matcher({"root_ids": root_id_list})],
         )
 
         qid_map = myclient.chunkedgraph.get_past_ids(
@@ -470,7 +550,7 @@ class TestChunkedgraph:
         url = chunkedgraph_endpoints_v1["handle_lineage_graph"].format_map(
             endpoint_mapping
         )
-        now = datetime.datetime.utcnow()
+        now = datetime.datetime.now(datetime.timezone.utc)
         timestamp_past = now - datetime.timedelta(days=7)
 
         query_d = package_timestamp(timestamp_past, name="timestamp_past")
@@ -621,7 +701,7 @@ class TestChunkedgraph:
             url=url,
             body=json.dumps(response_data),
             match=[
-                responses.json_params_matcher(
+                json_params_matcher(
                     {"sources": qdata_svid["sources"], "sinks": qdata_svid["sinks"]}
                 )
             ],
@@ -798,3 +878,34 @@ class TestChunkedgraph:
 
         base_resolution = myclient.chunkedgraph.base_resolution
         assert np.all(base_resolution == [8, 8, 40])
+
+    @responses.activate
+    def test_is_valid_nodes(self, myclient):
+        endpoint_mapping = self._default_endpoint_map
+        url = chunkedgraph_endpoints_v1["valid_nodes"].format_map(endpoint_mapping)
+        query_nodes = [91070075234304972, 91070075234296549]
+        data = {"node_ids": query_nodes}
+        return_data = {"valid_roots": query_nodes}
+        responses.add(
+            responses.GET,
+            status=200,
+            url=url,
+            json=return_data,
+            match=[json_params_matcher(data)],
+        )
+
+        out = myclient.chunkedgraph.is_valid_nodes(query_nodes)
+        assert np.all(out)
+
+        query_nodes = [0, -1]
+        data = {"node_ids": [0, 18446744073709551615]}
+        return_data = {"valid_roots": []}
+        responses.add(
+            responses.GET,
+            status=200,
+            url=url,
+            json=return_data,
+            match=[json_params_matcher(data)],
+        )
+        out = myclient.chunkedgraph.is_valid_nodes(query_nodes)
+        assert not np.any(out)
